@@ -11,7 +11,7 @@ import Yesod.Test
 import Yesod.Test.CssQuery
 import Yesod.Test.TransversingCSS
 import Text.XML
-import Data.Text (Text)
+import Data.Text (Text, pack)
 import Data.Monoid ((<>))
 import Control.Applicative
 import Network.Wai (pathInfo)
@@ -40,6 +40,22 @@ main = hspec $ do
             let html = "<html><head><title>foo</title></head><body><br><p>Hello World</p></body></html>"
                 query = "body > p"
              in findBySelector_ html query @?= ["<p>Hello World</p>"]
+        let query = "form.foo input[name=_token][type=hidden][value]"
+            html = "<input name='_token' type='hidden' value='foo'><form class='foo'><input name='_token' type='hidden' value='bar'></form>"
+            expected = "<input name=\"_token\" type=\"hidden\" value=\"bar\" />"
+         in it query $ findBySelector_ html (pack query) @?= [expected]
+        it "descendents and children" $
+            let html = "<html><p><b><i><u>hello</u></i></b></p></html>"
+                query = "p > b u"
+             in findBySelector_ html query @?= ["<u>hello</u>"]
+        it "hyphenated classes" $
+            let html = "<html><p class='foo-bar'><b><i><u>hello</u></i></b></p></html>"
+                query = "p.foo-bar u"
+             in findBySelector_ html query @?= ["<u>hello</u>"]
+        it "descendents" $
+            let html = "<html><p><b><i>hello</i></b></p></html>"
+                query = "p i"
+             in findBySelector_ html query @?= ["<i>hello</i>"]
     describe "HTML parsing" $ do
         it "XHTML" $
             let html = "<html><head><title>foo</title></head><body><p>Hello World</p></body></html>"
@@ -106,7 +122,7 @@ main = hspec $ do
                     setUrl ("/form" :: Text)
                     byLabel "Some Label" "12345"
                     fileByLabel "Some File" "test/main.hs" "text/plain"
-                    addNonce
+                    addToken
                 statusIs 200
                 bodyEquals "12345"
             yit "finding html" $ do
@@ -145,7 +161,7 @@ main = hspec $ do
             get ("/" :: Text)
             statusIs 200
             post ("/cookie/foo" :: Text)
-            statusIs 302
+            statusIs 303
             get ("/" :: Text)
             statusIs 200
             printBody
